@@ -3,8 +3,9 @@ use std::cmp::max;
 use std::error::Error;
 use wrecked::{RectManager, Color, WreckedError};
 
-use super::sbyte_editor::*;
-use super::sbyte_editor::converter::*;
+use super::shell::Shell;
+use super::editor::*;
+use super::editor::converter::*;
 
 use usize as RectId;
 
@@ -31,7 +32,10 @@ pub struct FrontEnd {
     input_context: String, // things may be displayed differently based on context
     rerow_flag: bool,
 
-    rect_help_window: RectId
+    rendered_buffer: Option<String>,
+
+    rect_help_window: RectId,
+    flag_context_changed: bool
 }
 
 impl FrontEnd {
@@ -45,6 +49,7 @@ impl FrontEnd {
         let rect_offset = rectmanager.new_rect(rect_meta).ok().unwrap();
         let rect_scrollbar = rectmanager.new_rect(rect_display_wrapper).ok().unwrap();
         let rect_help_window = rectmanager.new_rect(wrecked::ROOT).ok().unwrap();
+        rectmanager.detach(rect_help_window);
 
         let mut frontend = FrontEnd {
             rectmanager,
@@ -65,8 +70,10 @@ impl FrontEnd {
             last_known_viewport_offset: 9999,
 
             input_context: "DEFAULT".to_string(),
-            rerow_flag: false
+            rerow_flag: false,
+            rendered_buffer: None,
 
+            flag_context_changed: false
         };
 
 
@@ -78,18 +85,19 @@ impl FrontEnd {
     }
 
     pub fn set_input_context(&mut self, new_context: &str) {
+        self.flag_context_changed = true;
         self.input_context = new_context.to_string();
     }
 
-    pub fn tick(&mut self, sbyte_editor: &mut BackEnd) -> Result<(), Box::<dyn Error>> {
-        if !sbyte_editor.is_loading() {
-            let changed_viewport_size = sbyte_editor.has_changed("VIEWPORT_SIZE");
-            let changed_viewport_offset = sbyte_editor.has_changed("VIEWPORT_OFFSET");
-            let changed_cursor = sbyte_editor.has_changed("CURSOR");
-            let changed_cmdline = sbyte_editor.has_changed("CMDLINE");
+    pub fn tick(&mut self, shell: &mut Shell) -> Result<(), Box::<dyn Error>> {
+        let editor = shell.get_editor_mut();
+        if !editor.is_loading() {
+            let changed_viewport_size = editor.has_changed("VIEWPORT_SIZE");
+            let changed_viewport_offset = editor.has_changed("VIEWPORT_OFFSET");
+            let changed_cursor = editor.has_changed("CURSOR");
 
             if changed_viewport_size {
-                match self.setup_displays(sbyte_editor) {
+                match self.setup_displays(editor) {
                     Ok(_) => {}
                     Err(error) => {
                         Err(SbyteError::SetupFailed(error))?
@@ -98,7 +106,7 @@ impl FrontEnd {
             }
 
             if changed_viewport_size || changed_viewport_offset {
-                match self.remap_active_rows(sbyte_editor) {
+                match self.remap_active_rows(editor) {
                     Ok(_) => {}
                     Err(error) => {
                         Err(SbyteError::RemapFailed(error))?
@@ -106,10 +114,10 @@ impl FrontEnd {
                 }
             }
 
-            let changed_offsets = sbyte_editor.fetch_changed_offsets();
+            let changed_offsets = editor.fetch_changed_offsets();
             if !changed_offsets.is_empty() {
-                let (viewport_width, viewport_height) = sbyte_editor.get_viewport_size();
-                let viewport_bottom = (sbyte_editor.get_viewport_offset() / viewport_width) + viewport_height;
+                let (viewport_width, viewport_height) = editor.get_viewport_size();
+                let viewport_bottom = (editor.get_viewport_offset() / viewport_width) + viewport_height;
                 for (i, rippled) in changed_offsets.iter() {
                     let start_row = i / viewport_width;
                     if *rippled && viewport_bottom >= start_row {
@@ -125,7 +133,7 @@ impl FrontEnd {
             if !self.rows_to_refresh.is_empty() {
                 let tmp_rows_to_refresh: Vec<usize> = self.rows_to_refresh.drain().collect();
                 for y in tmp_rows_to_refresh.iter() {
-                    match self.set_row_characters(sbyte_editor, *y) {
+                    match self.set_row_characters(editor, *y) {
                         Ok(_) => {}
                         Err(error) => {
                             Err(SbyteError::RowSetFailed(error))?
@@ -136,7 +144,7 @@ impl FrontEnd {
 
 
             if changed_cursor || changed_viewport_size || changed_viewport_offset {
-                match self.apply_cursor(sbyte_editor) {
+                match self.apply_cursor(editor) {
                     Ok(_) => {}
                     Err(error) => {
                         Err(SbyteError::ApplyCursorFailed(error))?
@@ -144,34 +152,34 @@ impl FrontEnd {
                 }
             }
 
-
-            if changed_cmdline {
-                self.display_command_line(sbyte_editor)?;
+            if changed_cursor {
+                self.display_user_offset(editor)?;
             }
 
-            match sbyte_editor.get_user_error_msg() {
+
+            let mut feedback_or_error = false;
+            match shell.fetch_feedback() {
+                Some(msg) => {
+                    self.display_user_message(msg.clone())?;
+                    feedback_or_error = true;
+                }
+                None => { }
+            }
+
+            match shell.fetch_error() {
                 Some(msg) => {
                     self.display_user_error(msg.clone())?;
+                    feedback_or_error = true;
                 }
-                None => {
-                    match sbyte_editor.get_user_msg() {
-                        Some(msg) => {
-                            self.display_user_message(msg.clone())?;
-                        }
-                        None => {
-                            if !changed_cmdline && changed_cursor {
-                                self.clear_feedback()?;
-                            }
-                        }
-                    }
-                }
+                None => { }
             }
 
-            if changed_cursor {
-                self.display_user_offset(sbyte_editor)?;
 
+            if !feedback_or_error && (changed_cursor || self.flag_context_changed) {
+                self.clear_feedback()?;
             }
 
+            self.display_command_line(shell);
 
 
             match self.rectmanager.render() {
@@ -180,6 +188,8 @@ impl FrontEnd {
                     Err(SbyteError::DrawFailed(error))?;
                 }
             }
+
+            self.flag_context_changed = false;
         }
 
         Ok(())
@@ -189,12 +199,12 @@ impl FrontEnd {
         self.rectmanager.auto_resize()
     }
 
-    fn remap_active_rows(&mut self, sbyte_editor: &BackEnd) -> Result<(), WreckedError> {
+    fn remap_active_rows(&mut self, editor: &Editor) -> Result<(), WreckedError> {
 
-        let (width, height) = sbyte_editor.get_viewport_size();
+        let (width, height) = editor.get_viewport_size();
 
         let initial_y = self.last_known_viewport_offset as isize;
-        let new_y = (sbyte_editor.get_viewport_offset() / width) as isize;
+        let new_y = (editor.get_viewport_offset() / width) as isize;
         self.last_known_viewport_offset = new_y as usize;
 
         let diff: usize;
@@ -352,12 +362,12 @@ impl FrontEnd {
         Ok(())
     }
 
-    fn setup_displays(&mut self, sbyte_editor: &BackEnd) -> Result<(), WreckedError> {
+    fn setup_displays(&mut self, editor: &Editor) -> Result<(), WreckedError> {
         // Assumes that the viewport size AND the rectmanager size are correctly set at this point
         let full_width = self.rectmanager.get_width();
         let full_height = self.rectmanager.get_height();
 
-        let (viewport_width, viewport_height) = sbyte_editor.get_viewport_size();
+        let (viewport_width, viewport_height) = editor.get_viewport_size();
 
         self.rectmanager.resize(self.rect_meta, full_width, 1)?;
         self.rectmanager.resize(
@@ -377,12 +387,12 @@ impl FrontEnd {
         self.rectmanager.clear_children(bits_display)?;
         self.rectmanager.clear_children(human_display)?;
 
-        self.arrange_displays(sbyte_editor)?;
+        self.arrange_displays(editor)?;
 
         self.cell_dict.drain();
         self.row_dict.drain();
 
-        let display_ratio = sbyte_editor.get_display_ratio() as usize;
+        let display_ratio = editor.get_display_ratio() as usize;
         let width_bits;
         if display_ratio != 1 {
             width_bits = max(1, display_ratio - 1);
@@ -463,7 +473,7 @@ impl FrontEnd {
         Ok(())
     }
 
-    fn arrange_displays(&mut self, sbyte_editor: &BackEnd) -> Result<(), WreckedError> {
+    fn arrange_displays(&mut self, editor: &Editor) -> Result<(), WreckedError> {
         let full_width = self.rectmanager.get_width();
         let full_height = self.rectmanager.get_height();
         let meta_height = 1;
@@ -490,8 +500,8 @@ impl FrontEnd {
             0
         )?;
 
-        let display_ratio = sbyte_editor.get_display_ratio();
-        let (vwidth, _vheight) = sbyte_editor.get_viewport_size();
+        let display_ratio = editor.get_display_ratio();
+        let (vwidth, _vheight) = editor.get_viewport_size();
 
         let (bits_id, human_id) = self.rects_display;
         let human_display_width = vwidth;
@@ -517,14 +527,14 @@ impl FrontEnd {
         Ok(())
     }
 
-    fn set_row_characters(&mut self, sbyte_editor: &BackEnd, absolute_y: usize) -> Result<(), WreckedError> {
+    fn set_row_characters(&mut self, editor: &Editor, absolute_y: usize) -> Result<(), WreckedError> {
         let human_converter = HumanConverter {};
-        let active_converter = sbyte_editor.get_active_converter();
-        let (width, _height) = sbyte_editor.get_viewport_size();
+        let active_converter = editor.get_active_converter();
+        let (width, _height) = editor.get_viewport_size();
         let offset = width * absolute_y;
 
-        let chunk = sbyte_editor.get_chunk(offset, width);
-        let relative_y = absolute_y - (sbyte_editor.get_viewport_offset() / width);
+        let chunk = editor.get_chunk(offset, width);
+        let relative_y = absolute_y - (editor.get_viewport_offset() / width);
 
         match self.cell_dict.get_mut(&relative_y) {
             Some(cellhash) => {
@@ -585,10 +595,10 @@ impl FrontEnd {
         Ok(())
     }
 
-    pub fn display_user_offset(&mut self, sbyte_editor: &BackEnd) -> Result<(), WreckedError> {
-        let mut cursor_string = format!("{}", sbyte_editor.get_cursor_offset());
-        let active_content = sbyte_editor.get_active_content();
-        let (viewport_width, viewport_height) = sbyte_editor.get_viewport_size();
+    pub fn display_user_offset(&mut self, editor: &Editor) -> Result<(), WreckedError> {
+        let mut cursor_string = format!("{}", editor.get_cursor_offset());
+        let active_content = editor.get_active_content();
+        let (viewport_width, viewport_height) = editor.get_viewport_size();
 
         if active_content.len() > 0 {
             let digit_count = (active_content.len() as f64).log10().ceil() as usize;
@@ -607,7 +617,7 @@ impl FrontEnd {
             active_content.len() - 1
         };
 
-        let cursor_len = sbyte_editor.get_cursor_length();
+        let cursor_len = editor.get_cursor_length();
         let offset_display;
         if cursor_len == 1 {
             offset_display = format!("Offset: {} / {}", cursor_string, denominator)
@@ -627,7 +637,7 @@ impl FrontEnd {
             self.rectmanager.set_fg_color(handle, wrecked::Color::BLACK);
             self.rectmanager.resize(handle, 1, handle_height);
 
-            let handle_y = (sbyte_editor.get_viewport_offset() * (scrollbar_height - handle_height)) / (denominator - (viewport_width * viewport_height));
+            let handle_y = (editor.get_viewport_offset() * (scrollbar_height - handle_height)) / (denominator - (viewport_width * viewport_height));
             self.rectmanager.set_position(handle, 0, handle_y as isize);
         } else {
             self.rectmanager.disable(self.rect_scrollbar);
@@ -672,7 +682,7 @@ impl FrontEnd {
         Ok(())
     }
 
-    pub fn _unapply_cursor(&mut self, sbyte_editor: &BackEnd) -> Result<(), WreckedError> {
+    pub fn _unapply_cursor(&mut self, editor: &Editor) -> Result<(), WreckedError> {
         // (They may no longer exist, but that's ok)
         for (bits, human) in self.active_cursor_cells.drain() {
             self.rectmanager.clear_children(bits);
@@ -683,14 +693,14 @@ impl FrontEnd {
         Ok(())
     }
 
-    pub fn apply_cursor(&mut self, sbyte_editor: &BackEnd) -> Result<(), WreckedError> {
-        let (viewport_width, viewport_height) = sbyte_editor.get_viewport_size();
-        let viewport_offset = sbyte_editor.get_viewport_offset();
-        let cursor_offset = sbyte_editor.get_cursor_offset();
-        let cursor_length = sbyte_editor.get_cursor_length();
+    pub fn apply_cursor(&mut self, editor: &Editor) -> Result<(), WreckedError> {
+        let (viewport_width, viewport_height) = editor.get_viewport_size();
+        let viewport_offset = editor.get_viewport_offset();
+        let cursor_offset = editor.get_cursor_offset();
+        let cursor_length = editor.get_cursor_length();
 
         // First clear previously applied
-        self._unapply_cursor(sbyte_editor);
+        self._unapply_cursor(editor);
 
         let start = if cursor_offset < viewport_offset {
             viewport_offset
@@ -729,8 +739,8 @@ impl FrontEnd {
 
         match self.input_context.as_str() {
             "OVERWRITE_ASCII" | "OVERWRITE_HEX" | "OVERWRITE_BIN" | "OVERWRITE_DEC" => {
-                let subcursor_length = sbyte_editor.get_subcursor_length();
-                let subcursor_offset = sbyte_editor.get_subcursor_offset();
+                let subcursor_length = editor.get_subcursor_length();
+                let subcursor_offset = editor.get_subcursor_offset();
                 let suboffset_cell = cursor_offset + (subcursor_offset / subcursor_length);
 
                 y = (suboffset_cell - viewport_offset) / viewport_width;
@@ -768,30 +778,24 @@ impl FrontEnd {
         Ok(())
     }
 
-    pub fn display_command_line(&mut self, sbyte_editor: &BackEnd) -> Result<(), WreckedError> {
-        match sbyte_editor.get_commandline() {
-            Some(commandline) => {
-                self.clear_feedback()?;
+    pub fn display_command_line(&mut self, shell: &Shell) -> Result<(), WreckedError> {
+        if self.rendered_buffer != shell.buffer_get() {
+            match shell.buffer_get() {
+                Some(buffer) => {
+                    self.clear_feedback()?;
 
+                    let cursor_x = buffer.len() + 1;
+                    let cursor_id = self.rectmanager.new_rect(self.rect_feedback).ok().unwrap();
 
-                let cmd = &commandline.get_register();
-                // +1, because of the ":" at the start
-                let cursor_x = commandline.get_cursor_offset() + 1;
-                let cursor_id = self.rectmanager.new_rect(self.rect_feedback).ok().unwrap();
+                    self.rectmanager.resize(cursor_id, 1, 1)?;
+                    self.rectmanager.set_position(cursor_id, cursor_x as isize, 0)?;
+                    self.rectmanager.set_invert_flag(cursor_id)?;
 
-                self.rectmanager.resize(cursor_id, 1, 1)?;
-                self.rectmanager.set_position(cursor_id, cursor_x as isize, 0)?;
-                self.rectmanager.set_invert_flag(cursor_id)?;
-
-                if cursor_x < cmd.len() {
-                    let chr: String = cmd.chars().skip(cursor_x).take(1).collect();
-                    self.rectmanager.set_string(cursor_id, 0, 0, &chr)?;
+                    self.rectmanager.set_string(self.rect_feedback, 0, 0, &vec![":", &buffer].join(""))?;
                 }
-
-                self.rectmanager.set_string(self.rect_feedback, 0, 0, &vec![":", cmd].join(""))?;
-            }
-            None => {
+                None => { }
             }
+            self.rendered_buffer = shell.buffer_get();
         }
 
         Ok(())
