@@ -2,28 +2,27 @@ use std::env;
 use std::path::Path;
 use std::error::Error;
 
-pub mod sbyte_editor;
+pub mod editor;
 pub mod input_interface;
 pub mod console_displayer;
+pub mod shell;
 
-use sbyte_editor::*;
 use input_interface::InputInterface;
 use console_displayer::FrontEnd;
+use shell::Shell;
 
 fn result_catcher() -> Result<(), Box<dyn Error>> {
     let args: Vec<String> = env::args().collect();
 
-    let mut backend = BackEnd::new();
+    let mut shell = Shell::new();
     match args.get(1) {
         Some(path) => {
-            backend.load_file(path)?;
+            shell.get_editor_mut().load_file(path)?;
         }
         None => { }
     }
-
     let frontend = FrontEnd::new();
-
-    let mut input_interface = InputInterface::new(backend, frontend);
+    let mut input_interface = InputInterface::new(shell, frontend);
 
     // commands like setcmd run in custom_rc will overwrite whatever was set in the default
     let custom_rc_path = &format!("{}/.sbyterc", env::var("HOME").ok().unwrap());
@@ -39,9 +38,9 @@ fn result_catcher() -> Result<(), Box<dyn Error>> {
 
 fn main() {
     match result_catcher() {
-            Ok(_) => {}
-            Err(error) => {
-                println!("Fatal error {}", error);
-            }
+        Ok(_) => {}
+        Err(error) => {
+            eprintln!("Fatal error {}", error);
+        }
     }
 }
